

/*
Christof Ermer
17.03.2020
*/

#include <ctype.h>
#include <avr\pgmspace.h>
#include "main.h"

//#include "MorseTabelle_8Bit.h"
#include "MorseTx.h"
#include "Morse_8Bit_Simpel.h"
#include "RingBuf.h"

#ifdef MORSE_TX_USED

#define  TESTTEXT "Achtung Achtung, hier spricht die Polizei"

//GLOBAL 
MORSE_TYPE gstMorseTx;
uint8_t gu8MorseTxV;

//Proto
void Start_72KHZ(void);
void SetTimer1Frq( uint16_t u16Frq);
void Fetch_MorseTx(uint8_t u8Byte);
void ScanTxByte(void);
void Clear_PSP_Flags(void);
void ClearAllFlags(void);
void TriggerMorseBuf(void);

/*
// ******************************
ISR( TIMER2_COMPA_vect ) // 72KHz
// ******************************
{
static volatile uint16_t u16T2_Tick= 0;
if(gstMorseTx.Morse_SoundOn)
	{
	if(gstMorseTx.TX_Punkt)
		{
		if( u16T2_Tick > BEEP_PUNKT_F) {u16T2_Tick=0; TOGGLE_SOUND_TX();};
		u16T2_Tick++;
		};
	if(gstMorseTx.TX_Strich)
		{
		if( u16T2_Tick > BEEP_STRICH_F) {u16T2_Tick=0; TOGGLE_SOUND_TX();};
		u16T2_Tick++;
		};
	};
};
*/

// ******************************
void Init_MorseTx(void)
// ******************************
{
ClearAllFlags();
INIT_MORSETX_DATA();
//INIT_SOUND_TX();
INIT_BUSY_TX();
CLR_TXDATA();
//CLR_SOUND_TX();
CLR_BUSY_TX();
gstMorseTx.u16MorseSpeed = DEFAULT_MORSESPEED_POINT;
gstMorseTx.u16MorseTonFrq = BEEP_PUNKT_F;
gstMorseTx.u16SignalTxZeit = gstMorseTx.u16MorseSpeed;
Start_72KHZ();

gstFlags.AutoMorse_Flag = 0 ;
gstFlags.TX_Morse_Sofort_Flag = 0;
gstMorseTx.Morse_SoundOn = 1;
gstMorseTx.Morse_Busy = 0;

RingBuf_Init( &gsMorseBuf, gau8MorseStr, RINGBUFF_SIZE);
RingBuf_Flush( &gsMorseBuf );
};


// *********************************
void SetTimer1Frq( uint16_t u16Frq)
// *********************************
{
#define T1_VORTEILER 64.0
if(u16Frq)
	{
	DDRB |= (1<< DDB1);
	OCR1A = round( F_CPU / T1_VORTEILER / u16Frq / 2 ); //Toogle=*2
	TCNT1 = 0;
	TCCR1A = _BV(COM1A0);    //OC1A=PB1 Toogle
	//TCCR1B = _BV(WGM12) |  _BV(CS12) | _BV(CS10);  // /1024
	TCCR1B = _BV(WGM12) |  _BV(CS11) | _BV(CS10); //64
	}
else
	{
	TCCR1B = 0;
	PORTB &= ~(1<< DDB1);
	};	
};


 #ifdef MORSE_TX_USED
// ******************************
void ManageTxInterrupt(void)
// ******************************
{
/*
. = 1T
- = 3T
Lcke "ABC" = 1T
Wortlcke = 7T 
.(1T)(1T) -(3T)(1T)  
'R' = ENDE
*/
volatile static uint8_t u8CSelect=0;
volatile static uint16_t u16MorseTick=0;
//Toggle_Bit(1);

if( gstMorseTx.PSP_JobStart )  // 1*
	{	
	u16MorseTick = 0;
	u8CSelect = 0;
	gstMorseTx.PSP_JobStart = 0;
	gstMorseTx.PSP_JobActive = 1; //String
	gstMorseTx.Morse_Busy = 1;
	gstMorseTx.PSP_TXActive = 0; //Sendet gerade H/L  
	};
	
if( gstMorseTx.PSP_JobActive )
	{	
	if( gstMorseTx.PSP_TXActive == 0) //NOT
		{//Gerade am machen.
		switch( *(gstMorseTx.caTxJobStr + u8CSelect)  )
			{
			case 0 : // ende Str
				{
				*gstMorseTx.caTxJobStr = 0;
				ClearAllFlags();
				SetTimer1Frq( 0);
				CLR_MORSETX();	
				//CLR_SOUND_TX();	
				gstMorseTx.StrReady = 1;
				if( !RingBuf_Bytes_Available(&gsMorseBuf) ) 
					{
					gstMorseTx.Morse_Busy = 0;
					};				
				break;
				};
			case '.' :
				{		
				SET_MORSETX();
				Clear_PSP_Flags();
				gstMorseTx.u16SignalTxZeit = gstMorseTx.u16MorseSpeed;
				gstMorseTx.TX_Punkt = 1;				 
				SetTimer1Frq( BEEP_PUNKT_FRQ );
				gstMorseTx.PSP_TXActive = 1;
				break;
				};
			case '-' :
				{		
				SET_MORSETX();
				Clear_PSP_Flags();
				gstMorseTx.u16SignalTxZeit = gstMorseTx.u16MorseSpeed *3UL;
				gstMorseTx.TX_Strich = 1;
				SetTimer1Frq( BEEP_STRICH_FRQ );
				gstMorseTx.PSP_TXActive = 1;
				break;
				};
			case '1' : 
				{
				CLR_MORSETX();
				//CLR_SOUND_TX();						
				gstMorseTx.u16SignalTxZeit = gstMorseTx.u16MorseSpeed;
				Clear_PSP_Flags();
				gstMorseTx.TX_Pause = 1;
				gstMorseTx.PSP_TXActive = 1;				
				break;		
				};
			case '3' :
				{
				CLR_MORSETX();
				//CLR_SOUND_TX();						
				gstMorseTx.u16SignalTxZeit = gstMorseTx.u16MorseSpeed * 3UL;		
				Clear_PSP_Flags();
				gstMorseTx.TX_Pause = 1;
				gstMorseTx.PSP_TXActive = 1;				
				break;
				};
			case '7'  :
				{		
				CLR_MORSETX();
				//CLR_SOUND_TX();						
				gstMorseTx.u16SignalTxZeit = gstMorseTx.u16MorseSpeed * 7UL;
				Clear_PSP_Flags();
				gstMorseTx.TX_Pause = 1;
				gstMorseTx.PSP_TXActive = 1;				
				break;
				};				
			}; //Switch		
		}
	else  //PSP_TXActive 
		{
		if( u16MorseTick > gstMorseTx.u16SignalTxZeit )
			{			
			Clear_PSP_Flags();
			SetTimer1Frq(0);
			gstMorseTx.PSP_TXActive = 0;
			u16MorseTick = 0;
			u8CSelect++;
			CLR_MORSETX();	
			//CLR_SOUND_TX();		
			}
		else
			{
			u16MorseTick++;
			};
		};
	};//JobActive			
};
#endif // MORSE_TX_USED


//Punkt Srich Pause
// ******************************
void Clear_PSP_Flags(void)
// ******************************
{
gstMorseTx.TX_Pause = 0;
gstMorseTx.TX_Punkt = 0;
gstMorseTx.TX_Strich = 0;
};


// ******************************
void ClearAllFlags(void)
// ******************************
{
gstMorseTx.TXBYte_OK = 0;
gstMorseTx.PSP_JobStart = 0;
gstMorseTx.PSP_JobActive = 0;
gstMorseTx.PSP_TXActive = 0;
gstMorseTx.StrReady = 0;					
Clear_PSP_Flags();
};


// ******************************
void Start_72KHZ(void)
// ******************************
{
DDRB |= _BV(3); //PB3=OC2A OUT
TCNT2 = 0;
OCR2A = 222;
TCCR2A = _BV(COM2A0) | _BV(WGM21);  //CTC Toggle OC2A on Match
TCCR2B = _BV(CS20);  // /1
// TIMSK2 = _BV(OCIE2A); // Interupt   Kiern Ton auf dem Weg.. Knart, weil zuwenig Zeit
sei();
};


// ******************************
void Fetch_MorseTx( uint8_t u8Byte )
// ******************************
{
ClearAllFlags();
gstMorseTx.u8TxChar = u8Byte;
if( isalpha(u8Byte) ) 
	{	
	gstMorseTx.u8CodeOffset = CHAR_OFFEST + toupper(u8Byte) - 65; //A=65
	}
else
	{	
	if( isalnum(u8Byte) )
		{
		gstMorseTx.u8CodeOffset = u8Byte - 48; //48=0 #define ZAHL_OFFEST 0
		}
	else
		{
		if( isblank(u8Byte) )
			{
			gstMorseTx.u8CodeOffset = SPACE_OFFSET;	
			}
		else
			{ // war nix
			CLR_MORSETX();
//			CLR_SOUND_TX();
			ClearAllFlags();
			return;
			};
		};
	};
gstMorseTx.u8MorseVal  = pgm_read_byte_near( MCA_8BIT_SIMPELMORSE + gstMorseTx.u8CodeOffset );	
gstMorseTx.TXBYte_OK = 1;
};


// ******************************
void TxMorseStr(char * pxStr)
// ******************************
{
uint8_t u8NN=0;
while( (RingBuf_Check_Overload(&gsMorseBuf) == RING_OK) &&
	( *(pxStr+u8NN) ) && 
	(u8NN < (RINGBUFF_SIZE -1))	
)
	{
	RingBuf_WriteC( &gsMorseBuf, *(pxStr+u8NN) );
	u8NN++;
	};
};


// ******************************
void MorseBuf_Polling(void)
// ******************************
{
if( RingBuf_Bytes_Available(&gsMorseBuf) )
	{
	if( !(gstMorseTx.PSP_JobActive || gstMorseTx.PSP_JobStart) )
		{
		wdt_reset();
		TxMorseByte( RingBuf_ReadC( &gsMorseBuf) );
		SET_BUSY_TX();
		};
	}
else
	{
	if( !(gstMorseTx.PSP_JobActive || gstMorseTx.PSP_JobStart) )
		{
		CLR_BUSY_TX();
		gstMorseTx.Morse_Busy = 0;
		};
	};	
};


// ******************************
void TxMorseByte(uint8_t u8ScanVal)
// ******************************
{
uint8_t u8Pos=0;
Fetch_MorseTx( u8ScanVal );  //--> u8MorseVal

if( gstMorseTx.TXBYte_OK ) // alles OK
	{
	u8ScanVal = gstMorseTx.u8MorseVal; // nutze nun Variable fr shift	
	gstMorseTx.Morse_Busy = 1; /// !!!!!  BUSY   BUSY   BUSY ****

	if( u8ScanVal == 1 ) //" " SPACE
		{
		ClearAllFlags();
		strcpy_P( gstMorseTx.caTxJobStr, PSTR("7") );		
		gstMorseTx.PSP_JobStart = 1;
		}
	else
		{		
		while(u8ScanVal > 0x01)
			{
			if( u8ScanVal & 0x01 )  // Look last bit
				{ //STRICH
				*(gstMorseTx.caTxJobStr + u8Pos++) = '-';
				}
			else //PUNKT
				{
				*(gstMorseTx.caTxJobStr + u8Pos++) = '.';
				};	
		
			u8ScanVal = (u8ScanVal >> 1); // Kick >>
			if( u8ScanVal > 1 )
				{
				*(gstMorseTx.caTxJobStr + u8Pos++) = '1';
				}
			else
				{
				*(gstMorseTx.caTxJobStr + u8Pos++) = '3';
				};
			};
		}; //if 
	*(gstMorseTx.caTxJobStr + u8Pos) = 0;

//MINITO IST
#ifdef I2C_LCD_USED
	LCDShiftChars( gstMorseTx.u8TxChar );	
#endif
	uart_puts_p(PSTR("\r\nxChar:") );		
	uart_putc( gstMorseTx.u8TxChar );

	uart_puts_p(PSTR("\tMorseVal:") );		
	uart_puts( UIntToNumStr( gstMorseTx.u8MorseVal ) );

	uart_puts_p(PSTR("\tCodeOffs:") );			
	uart_puts( UIntToNumStr( gstMorseTx.u8CodeOffset ) );	

	uart_puts_p(PSTR("\tTxJobStr:") );		
	uart_puts( gstMorseTx.caTxJobStr );	
	
	gstMorseTx.PSP_JobStart = 1;
	
	}
else
	{
	ClearAllFlags();
	*gstMorseTx.caTxJobStr = 0; //clr
	};	
};

#endif